Package: panoramic
Title: Meta-Analysis of Spatial Colocalization in Spatial Omics
Version: 0.99.3
Authors@R: person("Jacob", "Chang", email = "jachang4@stanford.edu", role = c("aut", "cre", "fnd"), comment = c(ORCID = "0000-0002-3719-7949"))
Description: Provides a pipeline for quantifying and meta-analyzing
    spatial colocalization between cell types in spatial omics experiments.
    The package prepares SpatialExperiment inputs, computes Loh-bootstrap
    spatial summary functions (e.g. L- and K-functions) for cell-type pairs
    across samples, and performs random-effects meta-analysis to assess
    group-level differences in spatial colocalization.
URL: https://github.com/plevritis-lab/panoramic
BugReports: https://github.com/plevritis-lab/panoramic/issues
License: MIT + file LICENSE
Depends: R (>= 4.6)
Imports: dplyr, ggplot2, ggrepel, igraph, tidygraph, ggraph, rlang,
        S4Vectors, SummarizedExperiment, spatstat.geom,
        spatstat.explore, BiocParallel, SpatialExperiment, concaveman,
        metafor, withr, matrixStats, spatstat.utils
Suggests: knitr, rmarkdown, BiocStyle, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
biocViews: Software, Spatial, SingleCell
git_url: https://git.bioconductor.org/packages/panoramic
git_branch: devel
git_last_commit: ca3f3ca
git_last_commit_date: 2026-04-21
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
NeedsCompilation: no
Packaged: 2026-05-18 23:27:10 UTC; biocbuild
Author: Jacob Chang [aut, cre, fnd] (ORCID:
    <https://orcid.org/0000-0002-3719-7949>)
Maintainer: Jacob Chang <jachang4@stanford.edu>
