# PANORAMIC
**P**ooled **AN**alysis **O**f Va**R**iance-**A**ware **M**odeling and **I**nference of **C**olocalization (**PANORAMIC**)

[![DOI](https://zenodo.org/badge/1086842429.svg)](https://doi.org/10.5281/zenodo.17783978)

  <!-- badges: start -->
  [![R-CMD-check](https://github.com/plevritis-lab/panoramic/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/plevritis-lab/panoramic/actions/workflows/R-CMD-check.yaml)
  <!-- badges: end -->

This package is currently under development for submission to Bioconductor. 

![alt text](https://github.com/plevritis-lab/panoramic/blob/main/panoramic_graphic2.png)

## Installation
```r
if (!requireNamespace("BiocManager", quietly = TRUE)) {
  install.packages("BiocManager")
}
BiocManager::install("panoramic")
library(panoramic)
```

To install the development version from GitHub (optional):

```r
# install.packages("remotes")
remotes::install_github("plevritis-lab/panoramic")
```
