utils::globalVariables(c(
    "ct1", "ct2", "beta_diff", "p_diff", "fdr_diff",
    "neg_log10_p", "significance", "pair_label", "z_diff",
    "direction", "from", "to", "weight", "fdr", "pval",
    "estimate", "se", "tau2", "group", "type", "row_within_group",
    "group_offset", "y_pos", "ci_lower", "ci_upper", "label",
    "stats_text", "tau2_text", "marks", "cluster", "name",
    "x", "y", "x_value", "color_class", "feature_label",
    "edge_sig", "edge_class", "pnorm", "p.adjust"
))
