% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/panoramic-spatialstats.R
\name{.lohboot_local_comp}
\alias{.lohboot_local_comp}
\title{Internal: Loh-style bootstrap for local composition enrichment}
\usage{
.lohboot_local_comp(
  X,
  from,
  to,
  radii_um,
  nsim = 200,
  confidence = 0.95,
  type = 7,
  boot = c("approx", "block"),
  nx = 4,
  ny = nx,
  local_cache = NULL
)
}
\arguments{
\item{X}{A marked \code{ppp} object.}

\item{from, to}{Character marks defining anchor and target cell types.}

\item{radii_um}{Numeric radii at which enrichment is evaluated.}

\item{nsim}{Integer bootstrap replicates.}

\item{confidence}{Numeric confidence level.}

\item{type}{Quantile type for bootstrap intervals.}

\item{boot}{One of \code{"approx"} or \code{"block"}.}

\item{nx, ny}{Integer block counts (used only for \code{boot = "block"}).}

\item{local_cache}{Optional cache from \code{.precompute_local_comp_cache()}.}
}
\value{
A data.frame compatible with \code{.summarize_lohboot()}.
}
\description{
Internal: Loh-style bootstrap for local composition enrichment
}
\keyword{internal}
